'use strict';
var Curl = require( 'node-libcurl' ).Curl;
var FIFO = require('fifo');
var Promise = require("bluebird");

function CurlManage(options){
	if (!(this instanceof CurlManage))
    return new CurlManage(options);
	if("limit" in options){
		this._limit = options["limit"];
	}else{
		//default 10
		this._limit = 10;
	}
	if("no_data_parsing" in options){
		this._no_data_parsing = options["no_data_parsing"];
	}else{
		this._no_data_parsing = false;
	}
	this._current = 0;
	this._stack = FIFO();
}

CurlManage.prototype.request = function(options){
	var self = this;
	var curl = new Curl();
	//setOpt
	if(self._no_data_parsing) curl.enable(Curl.feature.NO_DATA_PARSING);
	Object.keys(options).map(function(key){
		/*if(Array.isArray(options[key])){
			options[key].map(function(opt){
				curl.setOpt(key,opt);
			});
		}else{
			curl.setOpt(key,options[key]);
		}*/
		curl.setOpt(key,options[key]);
	});
	if(self._limit>self._current){
		self._current++;
		return new Promise(function(resolve,reject){
			curl.on('end',function(statusCode,body,headers){
				var result = {
					statusCode: statusCode,
					body: body,
					headers: headers
				};
				self._current--;
				self.startPending();
				curl.close();
				resolve(result);
			});
			curl.on('error',function(err){
				self._current--;
				self.startPending();
				curl.close();
				reject(err);
			});
			curl.perform();
		});
	}else{
		self._stack.push(curl);
		return new Promise(function(resolve,reject){
			curl.on('end',function(statusCode,body,headers){
				var result = {
					statusCode: statusCode,
					body: body,
					headers: headers
				};
				self._current--;
				self.startPending();
				curl.close();
				resolve(result);
			});
			curl.on('error',function(err){
				self._current--;
				self.startPending();
				curl.close();
				reject(err);
			});
		});
	}
}

CurlManage.prototype.startPending = function(){
	var self = this;
	while(self._limit>self._current && self._stack.length != 0){
		self._current++;
		self._stack.shift().perform();
	}
}

module.exports = CurlManage;